/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2000-2007 Sun Microsystems, Inc. All rights reserved. 
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License ("CDDL") (collectively, the "License").  You may
 * not use this file except in compliance with the License.  You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or mq/legal/LICENSE.txt.  See the License for the specific language
 * governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at mq/legal/LICENSE.txt.  Sun designates
 * this particular file as subject to the "Classpath" exception as provided by
 * Sun in the GPL Version 2 section of the License file that accompanied this
 * code.  If applicable, add the following below the License Header, with the
 * fields enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or  to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright holder. 
 */

/*
 * @(#)PortMapperClient.hpp	1.4 06/26/07
 */ 

#ifndef PORTMAPPERCLIENT_HPP
#define PORTMAPPERCLIENT_HPP

#include "../debug/DebugUtils.h"
#include "../error/ErrorCodes.h"
#include "../basictypes/AllBasicTypes.hpp"
#include "../io/PortMapperTable.hpp"
#include "../containers/Properties.hpp"
#include "../util/PRTypesUtils.h"
#include "../basictypes/Object.hpp"

// The largest valid port number.
static const PRInt32  PORT_MAPPER_CLIENT_MAX_PORT_NUMBER         = MAX_PR_UINT16;

static const PRUint32 PORT_MAPPER_CLIENT_MAX_PORT_MAPPINGS_SIZE  = 2000;  // this was chosen arbitrarily

// Wait for 15 seconds to connect to the server
static const PRUint32 PORT_MAPPER_CLIENT_CONNECT_MICROSEC_TIMEOUT = 15 * 1000 * 1000;

// Wait for 15 seconds to receive information from the port server
static const PRUint32 PORT_MAPPER_CLIENT_RECEIVE_MICROSEC_TIMEOUT = 15 * 1000 * 1000;

/**
 * This class reads the MQ port mappings from the port mapper server
 * running on the broker.  
 */
class PortMapperClient : public Object {
private:
  
  PortMapperTable portMapperTable;

public:
  PortMapperClient();
  MQError readBrokerPorts(const Properties * const connectionProperties);
  MQError getPortForProtocol (const UTF8String * const protocol, 
                               const UTF8String * const type,
                                     PRUint16 *   const port);

//
// Avoid all implicit shallow copies.  Without these, the compiler
// will automatically define implementations for us.
//
private:
  //
  // These are not supported and are not implemented
  //
  PortMapperClient(const PortMapperClient& pmc);
  PortMapperClient& operator=(const PortMapperClient& pmc);
};

#endif // PORTMAPPERCLIENT_HPP
