/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2000-2007 Sun Microsystems, Inc. All rights reserved. 
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License ("CDDL") (collectively, the "License").  You may
 * not use this file except in compliance with the License.  You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or mq/legal/LICENSE.txt.  See the License for the specific language
 * governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at mq/legal/LICENSE.txt.  Sun designates
 * this particular file as subject to the "Classpath" exception as provided by
 * Sun in the GPL Version 2 section of the License file that accompanied this
 * code.  If applicable, add the following below the License Header, with the
 * fields enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or  to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright holder. 
 */

/*
 * @(#)ObjectVector.cpp	1.5 06/26/07
 */ 

#include "ObjectVector.hpp"
#include "../util/UtilityMacros.h"



// passing in false tells Vector to not call delete on the elements
// stored in the array.  We do this because calling delete on a void* pointer
// will not call the objects destructor.
ObjectVector::ObjectVector() : Vector(false)
{
  CHECK_OBJECT_VALIDITY();

}

ObjectVector::ObjectVector(PRUint32 initialSize) : Vector(initialSize, false)
{
  CHECK_OBJECT_VALIDITY();

}

ObjectVector::~ObjectVector()
{
  CHECK_OBJECT_VALIDITY();

  reset();
}

void
ObjectVector::reset()
{
  CHECK_OBJECT_VALIDITY();

  // Delete the individual elements of the Vector so the destructors of
  // the objects get called.
  while (size() > 0) {
    Object * element = NULL;
    pop((void**)&element);
    DELETE( element );
  }

  this->Vector::reset();
}
