/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2000-2007 Sun Microsystems, Inc. All rights reserved. 
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License ("CDDL") (collectively, the "License").  You may
 * not use this file except in compliance with the License.  You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or mq/legal/LICENSE.txt.  See the License for the specific language
 * governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at mq/legal/LICENSE.txt.  Sun designates
 * this particular file as subject to the "Classpath" exception as provided by
 * Sun in the GPL Version 2 section of the License file that accompanied this
 * code.  If applicable, add the following below the License Header, with the
 * fields enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or  to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright holder. 
 */

/*
 * @(#)Decompressor.java	1.3 06/27/07
 */ 

package com.sun.messaging.jmq.jmsclient.zip;

import java.util.zip.*;

import javax.jms.*;

import com.sun.messaging.jmq.jmsclient.Debug;

public class Decompressor {

    private Inflater inflater = new Inflater(false);

    private static Decompressor decompressor = new Decompressor();

    private boolean debug = Boolean.getBoolean("imq.zip.debug");

    private Decompressor () {
    }

    public static Decompressor getInstance() {
        return decompressor;
    }

    public synchronized void
    decompress (byte[] zipBody, byte[] unzipBody) throws JMSException {

        Inflater inf = getDefaultInflater();

        try {
            inf.setInput( zipBody );

            int uncompressedSize = inf.inflate( unzipBody );

            if ( uncompressedSize != unzipBody.length ) {
                //This should never happen!
                throw new JMSException (
                "Error occurred in decompression. unzip size: " +
                uncompressedSize + " expected size: " + unzipBody.length);
            }

            if ( debug ) {
                Debug.getPrintStream().println
                    ("*** decompressor zip size: " + zipBody.length);
                Debug.getPrintStream().println
                    ("*** decompressor unzip size: " + uncompressedSize);
            }

        } catch (DataFormatException e) {
            JMSException jmse = new JMSException (e.toString());
            jmse.setLinkedException(e);
            throw jmse;
        } finally {
            inf.reset();
        }
    }

    private Inflater getDefaultInflater() {
        return this.inflater;
    }

}
