/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2000-2007 Sun Microsystems, Inc. All rights reserved. 
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License ("CDDL") (collectively, the "License").  You may
 * not use this file except in compliance with the License.  You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or mq/legal/LICENSE.txt.  See the License for the specific language
 * governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at mq/legal/LICENSE.txt.  Sun designates
 * this particular file as subject to the "Classpath" exception as provided by
 * Sun in the GPL Version 2 section of the License file that accompanied this
 * code.  If applicable, add the following below the License Header, with the
 * fields enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or  to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright holder. 
 */

/*
 * @(#)TransactionID.java	1.9 06/28/07
 */ 

package com.sun.messaging.jmq.jmsserver.data;

import java.io.*;

public class TransactionID implements Serializable
{
    private int port = 0;
    private byte[] IP = null;
    private int transid = 0;
    private String unique_id = null;

    public TransactionID(int transid, byte[] IP, int port)
    {
        this.port = port;
        this.IP = IP;
        this.transid = transid;
    }

    private void initUniqueStr() {

        unique_id = "";
        
        for (int i = 0; i < IP.length; i ++) {
            unique_id +=Integer.toHexString(((int)IP[i]) & 0xff);
        }
        unique_id += ":" + Integer.toHexString(port) + ":" + Integer.toHexString(transid);
    }

    public boolean equals(Object obj)
    {
        if (! (obj instanceof TransactionID)) 
            return false;

        TransactionID tid = (TransactionID) obj;

        if (port != tid.port || transid != tid.transid) {
            return false;
        }

        if ((IP == null || tid.IP == null) && IP != tid.IP ) 
            return false;

        if (IP.length != tid.IP.length)
            return false;

        for (int i =0; i < IP.length; i ++)
            if (IP[i] != tid.IP[i])
                return false;
        return true;
    }

    public int hashCode() {
        int h = 0;
        for (int i =0; i < IP.length; i ++)
            h += (new Byte(IP[i])).intValue();

        h = 31*h + port;
        h = 31*h + transid;

        return h;
    }

    public String toString() {
/*
        return "TransactionID[" + transid + "," +
                 (new String(IP)) + "," + port + "]";
*/
        return "TransactionID[" + transid + "," +
                  port + "]";
    }

    public String getUniqueName() {
        if (unique_id == null) {
            initUniqueStr();
        }
        return unique_id;

    }

}
