/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2000-2007 Sun Microsystems, Inc. All rights reserved. 
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License ("CDDL") (collectively, the "License").  You may
 * not use this file except in compliance with the License.  You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or mq/legal/LICENSE.txt.  See the License for the specific language
 * governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at mq/legal/LICENSE.txt.  Sun designates
 * this particular file as subject to the "Classpath" exception as provided by
 * Sun in the GPL Version 2 section of the License file that accompanied this
 * code.  If applicable, add the following below the License Header, with the
 * fields enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or  to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright holder. 
 */

/*
 * @(#)GoodbyeHandler.java	1.9 06/28/07
 */ 

package com.sun.messaging.jmq.jmsserver.multibroker.raptor.handlers;

import java.io.*;
import java.util.Hashtable;
import com.sun.messaging.jmq.util.*;
import com.sun.messaging.jmq.jmsserver.util.*;
import com.sun.messaging.jmq.io.*;
import com.sun.messaging.jmq.jmsserver.Globals;
import com.sun.messaging.jmq.jmsserver.core.*;
import com.sun.messaging.jmq.jmsserver.multibroker.raptor.*;
import com.sun.messaging.jmq.jmsserver.resources.BrokerResources;

public class GoodbyeHandler extends GPacketHandler {
    private static boolean DEBUG = false;

    public GoodbyeHandler(RaptorProtocol p) {
        super(p);
    }

    public void handle(BrokerAddress sender, GPacket pkt) {
        if (pkt.getType() == ProtocolGlobals.G_GOODBYE) {
            handleGoodbye(sender, pkt);
        }
        else if (pkt.getType() == ProtocolGlobals.G_GOODBYE_REPLY) {
            handleGoodbyeReply(sender, pkt);
        }
        else {
            logger.log(logger.WARNING, br.E_INTERNAL_BROKER_ERROR,
                       "Cannot handle this packet :" + pkt.toLongString());
        }
    }

    public void handleGoodbye(BrokerAddress sender, GPacket pkt) {
         
        try {
            ClusterGoodbyeInfo cgi = ClusterGoodbyeInfo.newInstance(pkt, c);
            p.goodbyeReceived(sender, cgi);
            p.sendGoodbye(sender);
            cbDispatcher.processGoodbye(sender, cgi);
            if (cgi.needReply()) {
                GPacket gp = cgi.getReplyGPacket(ProtocolGlobals.G_SUCCESS);
                try {
                c.unicastAndClose(sender, gp);
                p.goodbyeReplySent(sender);
                } catch (IOException e) {
                  logger.logStack(logger.DEBUG, 
                    Globals.getBrokerResources().getKString(
                    BrokerResources.W_CLUSTER_UNICAST_FAILED, 
                    ProtocolGlobals.getPacketTypeDisplayString(gp.getType()), sender),e);
                }
            }
        }
        catch (Exception e) {
            logger.logStack(logger.INFO, br.E_INTERNAL_BROKER_ERROR,
                            "Unable to process packet: " + pkt, e);
            return;
        }
    }

    public void handleGoodbyeReply(BrokerAddress sender, GPacket pkt) {
        cbDispatcher.processGoodbyeReply(sender);
        p.goodbyeReplyReceived(sender);
    }

}
