/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2000-2007 Sun Microsystems, Inc. All rights reserved. 
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License ("CDDL") (collectively, the "License").  You may
 * not use this file except in compliance with the License.  You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or mq/legal/LICENSE.txt.  See the License for the specific language
 * governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at mq/legal/LICENSE.txt.  Sun designates
 * this particular file as subject to the "Classpath" exception as provided by
 * Sun in the GPL Version 2 section of the License file that accompanied this
 * code.  If applicable, add the following below the License Header, with the
 * fields enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or  to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright holder. 
 */

/*
 * @(#)SelectorFilter.java	1.12 06/29/07
 */ 

package com.sun.messaging.jmq.jmsserver.util;

import com.sun.messaging.jmq.jmsserver.core.PacketReference;
import com.sun.messaging.jmq.jmsserver.core.Consumer;
import com.sun.messaging.jmq.jmsserver.Globals;
import com.sun.messaging.jmq.util.lists.Filter;
import com.sun.messaging.jmq.util.log.*;
import com.sun.messaging.jmq.io.Packet;
import java.lang.ref.*;
import com.sun.messaging.jmq.util.selector.*;
import java.util.Hashtable;
import java.util.Map;
import java.io.IOException;

public class SelectorFilter implements Filter
{
    private static boolean DEBUG = false;
    Selector selector = null;
    String selectorstr = null;

    public SelectorFilter(String selectorstr) 
         throws SelectorFormatException
    {
        this.selectorstr = selectorstr;
        selector =  Selector.compile(selectorstr);
     
    }
    public SelectorFilter(String selectorstr, Selector sel) 
    {
        this.selectorstr = selectorstr;
        this.selector = sel;
    }

    public synchronized boolean matches(Object o) 
    {
        if (selector == null) {
            return false;
        }
        if (o instanceof PacketReference) {
            PacketReference ref = (PacketReference) o;
            Map props = null;
            Map headers = null;
            try {
                // As an optimization, only extract these if the
                // selector needs them.
                if (selector.usesProperties()) {
                   props = ref.getProperties();
                }
                if (selector.usesFields()) {
                    headers = ref.getHeaders();
                }
            } catch (ClassNotFoundException ex) {
                // this is not a valid error
                assert false : ref;
                throw new RuntimeException("error with properties",
                     ex);
            }
            try {
                boolean match =  selector.match(props, headers);
                if (DEBUG && match)
                    Globals.getLogger().log(Logger.DEBUG,"Match " 
                            + o + "against " + selector + " got " + match);
                return match;
            } catch (SelectorFormatException ex) {
                Globals.getLogger().log(Logger.INFO,"Bad selector format, can not match against  " 
                     + selector, ex);
                return false;
            }
        }
        assert false : " weird ";
        return false;
    }

    public String toString() {
        return "SelectorFilter["+selector+"]"+hashCode();
    }


}
