/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2000-2007 Sun Microsystems, Inc. All rights reserved. 
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License ("CDDL") (collectively, the "License").  You may
 * not use this file except in compliance with the License.  You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or mq/legal/LICENSE.txt.  See the License for the specific language
 * governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at mq/legal/LICENSE.txt.  Sun designates
 * this particular file as subject to the "Classpath" exception as provided by
 * Sun in the GPL Version 2 section of the License file that accompanied this
 * code.  If applicable, add the following below the License Header, with the
 * fields enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or  to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright holder. 
 */

/*
 * @(#)ServerSession.java	1.5 06/29/07
 */ 

package com.sun.messaging.jmq.jmsspi;

/**
 * A ConnectionConsumer implemented by a JMS provider uses a ServerSession
 * to process one or more messages that have arrived. It does this by getting
 * a ServerSession from the ConnectionConsumer's ServerSessionPool; getting
 * the ServerSession's JMS session; loading it with the messages; and then
 * starting the ServerSession.

 * ServerSession is an abstraction of the association between a JMS Session,
 * a thread and a message-bean instance. Each ServerSession given to a
 * ConnectionConsumer represents a single thread of message-delivery.
 *
 * This interface extends javax.jms.ServerSession 
 *
 */

public interface ServerSession extends javax.jms.ServerSession {

    /**
     * Hook to enable container processing just prior to msg delivery.
     * For example, allow a transaction to be started just prior to msg
     * being delivered to the Session MessageListener.
     *
     * To be called by the thread that invokes Session's 
     * MessageListener.onMessage()
     *
     * This call must be paired with the call afterMessageDelivery
     *
     * @param msg Message that is about to be delivered.
     */
    void beforeMessageDelivery(javax.jms.Message msg);

    /**
     * Hook to enable container processing after msg delivery.
     *
     * To be called by the thread that invokes Session's 
     * MessageListener.onMessage()
     *
     * @param msg Message that was delivered.
     */
    void afterMessageDelivery(javax.jms.Message msg);

    /**
     * Indicate that this ServerSession is invalidate and should not 
     * be used again. 
     *
     * To be called by ConnectionConsumer when finds the ServerSession
     * invalid (e.g. unable to getSession or its Session is invalid).
     *
     * This method must not be called after ServerSession.start() has
     * been successfully returned.
     */
    void destroy();
}

